from rest_framework.generics import ListAPIView, RetrieveUpdateDestroyAPIView, ListAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models.main_req import Main
from reqman.apps.reqtool.rest_api.serializers.main_req import MainSerializer

from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki

from rest_framework.response import Response
from rest_framework.views import APIView

from reqman.apps.reqtool.models.main_req import SYSTEM_CHOICES, FUNCTION_CHOICES, INTERFACE_CHOICES, STATE_CHOICES, ITEM_CHOICES
from reqman.apps.reqtool.models.main_req import STATE_VALUE_CHOICES, SHALL_CHOICES, VERB_CHOICES, QUANTIFIER_CHOICES, NUMBER_UNITS_CHOICES
from reqman.apps.reqtool.models.main_req import FLOW_CHOICES, CONNECTION_CHOICES, STATE_SET_CHOICES

from reqman.apps.reqtool.models.requirements import BoilerplateData

class MainListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = MainSerializer
    #
    queryset = Main.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_main = Main.objects.filter(boilerplate_of_main = group_pk)
        return queryset_main

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save()
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class MainDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]

    serializer_class = MainSerializer
    
    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_main= Main.objects.filter(boilerplate_of_main = group_pk)
        return queryset_main

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.validated_data
        #add data in thn Model BoilerplateData
        mn = (instance['sys_fun_inter'] + ' ' + instance['shall'] + ' ' + instance['verb'] + ' ' + instance['flow_function_interface_item_system_state_stateset'])
        BoilerplateData.objects.filter(owner_data=instance['boilerplate_of_main'].owner, boilerplate_data_id = instance['boilerplate_of_main']).update(main_data = mn )
        serializer.save()
        #print(self.new.changed_data)
        #instance = serializer.save()
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #instance = serializer.validated_data
        instance.delete()


class SysFunIntChoicesViewSet(APIView):

    def get(self, request):
        response = Response(SYSTEM_CHOICES + FUNCTION_CHOICES + INTERFACE_CHOICES)
        return response


class StatItemChoicesViewSet(APIView):

    def get(self, request):
        response = Response(STATE_CHOICES + ITEM_CHOICES)
        return response


class StatevalueChoicesViewSet(APIView):

    def get(self, request):
        print(STATE_VALUE_CHOICES)
        response = Response(STATE_VALUE_CHOICES)
        return response


class ShallChoicesViewSet(APIView):

    def get(self, request):
        response = Response(SHALL_CHOICES)
        return response


class VerbChoicesViewSet(APIView):

    def get(self, request):
        response = Response(VERB_CHOICES)
        return response


class QuantChoicesViewSet(APIView):

    def get(self, request):
        response = Response(QUANTIFIER_CHOICES)
        return response


class NumUnitChoicesViewSet(APIView):

    def get(self, request):
        response = Response(NUMBER_UNITS_CHOICES)
        return response


class FlFuIntItSysStatSetChoicesViewSet(APIView):

    def get(self, request):
        response = Response(FLOW_CHOICES + FUNCTION_CHOICES + INTERFACE_CHOICES + ITEM_CHOICES + SYSTEM_CHOICES +STATE_CHOICES + STATE_SET_CHOICES)
        return response


class StaSysConSetChoicesViewSet(APIView):

    def get(self, request):
        response = Response(STATE_VALUE_CHOICES + SYSTEM_CHOICES + CONNECTION_CHOICES + STATE_SET_CHOICES)
        return response
