from rest_framework.generics import ListCreateAPIView, RetrieveUpdateDestroyAPIView, ListAPIView
from rest_framework import permissions

from django.http import HttpResponse, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from rest_framework.parsers import JSONParser

from reqman.apps.reqtool.models.requirements import Prefix, Boilerplate, PREFIX_CHOICES, LOGIC_CONNECTIVE, VERB_CHOICES
from reqman.apps.reqtool.models.requirements import VERB_CHOICES, ITEM_CHOICES, SYSTEM_CHOICES, FUNCTION_CHOICES, STATE_CHOICES
from reqman.apps.reqtool.models.requirements import ITEM_CHOICES, FLOW_CHOICES, FUNCTION_CHOICES, STATE_VALUE_CHOICES
from reqman.apps.reqtool.rest_api.serializers.requirements import BoilerplateSerializer, PrefixSerializer, BoilerplateDataSerializer
from reqman.apps.permissions import IsOwnerOrReadOnly
from reqman.apps.reqtool.rest_api.services import fuseki
from rest_framework.response import Response
from rest_framework.views import APIView

from reqman.apps.reqtool.models.requirements import BoilerplateData
from django.shortcuts import get_object_or_404

class BoilerplateListCreateAPIView(ListCreateAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateSerializer
    #queryset = 

    def get_queryset(self):
        #print(title)
        queryset_boil = Boilerplate.objects.filter(owner = self.request.user)
        return queryset_boil

    #Custom actions when POST
    def perform_create(self, serializer):
        serializer.save(owner=self.request.user)
        #print the data of the post
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

class BoilerplateList(ListAPIView):
    """
    API view to retrieve list of posts based on title of Boilerplate
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateSerializer
    queryset = Boilerplate.objects.all()


    def get_queryset(self):
        title = self.kwargs['title_1']
        #print(title)
        queryset_title = Boilerplate.objects.filter(title = title)
        return queryset_title

   

class BoilerplateDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly,
                      IsOwnerOrReadOnly]
    serializer_class = BoilerplateSerializer
    queryset = Boilerplate.objects.all()

    #Custom actions when PUT
    def perform_update(self, serializer):
        #print(serializer.validated_data['id'])
        #if not Prefix.objects.filter(boilerplate = serializer.validated_data['id']) and (serializer.validated_data['b_prefix'] == True):
        #    serializer.validated_data['b_prefix'] = True
        instance = serializer.save()
        
        #write_fuseki = fuseki.FusekiActions()
        #write_fuseki.write(serializer.data)

    #Custom actions when DELETE
    def perform_destroy(self, instance):
        #print("deleted")
        instance.delete()


class PrefixListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = PrefixSerializer
    #
    #queryset = Prefix.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        #print(group_pk)
        queryset_prefix = Prefix.objects.filter(prefix_boilerplate = group_pk)
        return queryset_prefix

class PrefixDetailsAPIView(RetrieveUpdateDestroyAPIView):
    """
    API view to retrieve, update or delete post
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = PrefixSerializer
    #queryset = Prefix.objects.all()

    def get_queryset(self):
        group_pk = self.kwargs['boilerplate']
        queryset_prefix = Prefix.objects.filter(prefix_boilerplate = group_pk)
        return queryset_prefix

    def simple_prefix(self, prefix):
        if prefix == "If/Unless":
            return "P1"
        if prefix == "As soon as":
            return "P2"
        if prefix == "As long as":
            return "P3"
        

    #Custom actions when PUT
    def perform_update(self, serializer):
        instance = serializer.validated_data
        #add data in thn Model BoilerplateData
        pr = (instance['prefix'] + ' ' + instance['system_fun_item'] + ' ' + instance['state_or_verb'] + ' ' + instance['item_function_flow_statevalue'])
        BoilerplateData.objects.filter(owner_data=instance['prefix_boilerplate'].owner, boilerplate_data_id = instance['prefix_boilerplate']).update(prefix_data = pr )
        sprefix = self.simple_prefix(instance["prefix"])
        serializer.save(simple_prefix = sprefix)


    #Custom actions when DELETE
    def perform_destroy(self, instance):
        instance = serializer.validated_data
        #print("deleted")
        instance.delete()


class BoilerplateDataListAPIView(ListAPIView):
    """
    API view to retrieve list of posts or create new
    """
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]
    serializer_class = BoilerplateDataSerializer
    #queryset = Prefix.objects.all()

    def get_queryset(self):
        #print(title)
        queryset_boil_data = BoilerplateData.objects.filter(owner_data = self.request.user)
        return queryset_boil_data


class PrefixChoicesViewSet(APIView):

    def get(self, request):
        response = Response(PREFIX_CHOICES)
        return response

class LogConChoicesViewSet(APIView):

    def get(self, request):
        response = Response(LOGIC_CONNECTIVE)
        return response

class VerbChoicesViewSet(APIView):

    def get(self, request):
        response = Response(STATE_CHOICES + VERB_CHOICES)
        return response

class FunItemSysChoicesViewSet(APIView):

    def get(self, request):
        response = Response(ITEM_CHOICES + SYSTEM_CHOICES + FUNCTION_CHOICES)
        return response

class StaFlowFunItemChoicesViewSet(APIView):

    def get(self, request):
        response = Response(ITEM_CHOICES + FLOW_CHOICES + FUNCTION_CHOICES + STATE_VALUE_CHOICES)
        return response
